matrix = [
  {
    'PHP_VERSION': '5',
    'ALPINE_VERSION': ['3.8', '3.7'],
    'TESTING_REPO': 0,
    'PHP_SESSION': 0,
  },
  {
    'PHP_VERSION': '7',
    'ALPINE_VERSION': ['3.15', '3.14', '3.13', '3.12', '3.11', '3.10', '3.9', '3.8', '3.7'],
    'TESTING_REPO': 0,
    'PHP_SESSION': 1,
  },
  {
    'PHP_VERSION': '8',
    'TESTING_REPO': 0,
    'PHP_SESSION': 1,
    'ALPINE_VERSION': ['3.16', '3.15', '3.14', '3.13'],
  },
  {
    'PHP_VERSION': '81',
    'ALPINE_VERSION': ['edge', 'latest', '3.16', '3.17'],
    'TESTING_REPO': 0,
    'PHP_SESSION': 1,
  },
  {
    'PHP_VERSION': '82',
    'ALPINE_VERSION': ['edge'],
    'TESTING_REPO': 1,
    'PHP_SESSION': 1,
  }
  ]

def main(ctx):
  return [buildContainer(step['PHP_VERSION'], ALPINE_VERSION, step['TESTING_REPO'], step['PHP_SESSION']) for step in matrix for ALPINE_VERSION in step['ALPINE_VERSION']]

def buildContainer(PHP_VERSION, ALPINE_VERSION, TESTING_REPO, PHP_SESSION):
  return {
    'kind': 'pipeline',
    'type': 'docker',
    'name': 'php' + PHP_VERSION + '-alpine' + ALPINE_VERSION,
    'steps': [
      {
        'name': 'detect-image-tag',
        'image': 'archlinux',
        'commands': [
          '''if  [[ "$DRONE_BRANCH" == "master" ]]; then
                tag="latest";
                echo "Running on default branch '$DRONE_BRANCH': tag = 'latest'";
              else
                tag="$DRONE_BRANCH";
                echo "Running on branch '$DRONE_BRANCH': tag = $tag";
              fi''',
          'echo "tag=$tag" >> .env'
        ]
      },
      {
        'name': 'detect-registry-urls',
        'image': 'archlinux',
        'commands': [
          'echo "Repo: $DRONE_REPO"',
          'registry_image=$(echo "$DRONE_GIT_HTTP_URL" | awk -F "://" \'{print $2}\' | awk -F ".git" \'{print tolower($1)}\')',
          'registry_domain=$(echo "$registry_image" | awk -F "/" \'{print $1}\')',
          'echo "registry_image=$registry_image" >> .env',
          'echo "registry_domain=$registry_domain" >> .env'
        ]
      },
      {
        'name': 'login to registry',
        'image': 'docker:dind',
        'volumes': [
          {
            'name': 'docker',
            'path': '/root/.docker'
          }
        ],
        'environment': {
          'DOCKER_PASSWORD': {
            'from_secret': 'docker_password'
          }
        },
        'commands': [
          'source .env',
          'cat .env',
          'echo $DOCKER_PASSWORD | docker login --username $DRONE_REPO_OWNER --password-stdin $registry_domain'
        ]
      },
      {
        'name': 'wait for docker',
        'image': 'archlinux',
        'volumes': [
          {
            'name': 'sockets',
            'path': '/var/run'
          },
        ],
        'commands': [
          '''while [ ! -S /var/run/docker.sock ]; do
                echo "Waiting for docker socket..."
                sleep 1
              done'''
        ]
      },
      {
        'name': 'build image',
        'image': 'docker:dind',
        'volumes': [
          {
            'name': 'sockets',
            'path': '/var/run'
          },
          {
            'name': 'docker',
            'path': '/root/.docker'
          }
        ],
        'commands': [
          'source .env',
          'cat .env',
          'docker context create build',
          'docker buildx create build --use',
          'docker buildx build --squash --platform linux/amd64,linux/arm,linux/arm64 --push --build-arg PHP_VERSION=' + PHP_VERSION + ' --build-arg ALPINE_VERSION=' + ALPINE_VERSION + ' --build-arg TESTING_REPO=' + str(TESTING_REPO) + ' --build-arg PHP_SESSION=' + str(PHP_SESSION) + ' -t $registry_image:${tag}' + PHP_VERSION + '-' + ALPINE_VERSION + ' .'
        ]
      }
    ],
    'services': [
      {
        'name': 'docker',
        'image': 'docker:dind',
        'privileged': True,
        'environment': {
          'DOCKER_TLS_CERTDIR': '/certs'
        },
        'volumes': [
          {
            'name': 'sockets',
            'path': '/var/run'
          }
        ]
      }
    ],
    'volumes': [
      {
        'name': 'sockets',
        'temp': {}
      },
      {
        'name': 'docker',
        'temp': {}
      }
    ],
  }